"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var zlib = __importStar(require("zlib"));
var utils_1 = require("./utils");
var follow_redirects_1 = require("follow-redirects");
/**
 * In nodejs environment, by default for XMLHttpRequest, fetch and wx.request, http-request-mock
 * does a fallback request by simply using http/https native request module, which encodes the
 * response body with utf8. It may not meet your requirement in some complex applications.
 * So, you can use another third fake (XMLHttpRequest, fetch, wx.request)request library
 * instead before calling setupForUnitTest method if you had some problems with the fallback request.
 *
 * @param {string} url
 * @param {string} method
 * @param {object} headers
 * @param {any} body
 * @param {object} opts
 */
function request(requestConfig) {
    var url = requestConfig.url, method = requestConfig.method, _a = requestConfig.headers, headers = _a === void 0 ? {} : _a, body = requestConfig.body, _b = requestConfig.opts, opts = _b === void 0 ? {} : _b;
    return new Promise(function (resolve, reject) {
        var isHttps = isHttpsUrl(url);
        var reqOpts = __assign({ useNativeModule: true, method: (method || 'GET').toUpperCase(), headers: headers || {} }, opts);
        var requestMethod = (isHttps ? follow_redirects_1.https : follow_redirects_1.http).request;
        var req = requestMethod(url, reqOpts, function (response) {
            getResponseBody(response).then(function (_a) {
                var body = _a.body, json = _a.json;
                resolve({ body: body, json: json, response: response });
            }).catch(function (err) {
                req.emit('error', err);
            });
        });
        req.on('error', function (err) {
            reject(err);
        });
        if (body) {
            if (typeof body === 'string'
                || (body instanceof Buffer)
                || (body instanceof ArrayBuffer)
                || (body instanceof SharedArrayBuffer)
                || (body instanceof Uint8Array)) {
                req.write(Buffer.from(body));
            }
            else {
                req.write(Buffer.from(JSON.stringify(body)));
            }
        }
        req.end();
    });
}
exports.default = request;
function isHttpsUrl(url) {
    if (typeof url === 'string') {
        return /^https:/i.test(url);
    }
    if (url && (url.href || url.protocol)) {
        return /^https:/i.test(url.href) || String(url.protocol).toLowerCase() === 'https:';
    }
    return false;
}
function getResponseBody(response) {
    var stream;
    if (['gzip', 'compress', 'deflate'].includes(response.headers['content-encoding'] || '')) {
        stream = response.pipe(zlib.createGunzip());
    }
    else if ('br' === response.headers['content-encoding']) {
        stream = response.pipe(zlib.createBrotliDecompress());
    }
    else {
        stream = response;
    }
    return new Promise(function (resolve, reject) {
        stream.once('error', function (error) {
            stream.removeAllListeners();
            reject(error);
        });
        var buf = [];
        stream.on('data', function (chunk) { return buf.push(chunk); });
        stream.once('end', function () {
            var body = Buffer.concat(buf).toString();
            resolve({ body: body, json: (0, utils_1.tryToParseJson)(body, null) });
            stream.removeAllListeners();
        });
    });
}
//# sourceMappingURL=request.js.map